require_relative '../itjobstack.rb'
require 'faraday'
require 'json'

# @author Frank Pimenta
module Itjobstack
  module API
    # handles api exceptions
    class Exception < ::Exception
      def initialize message
        @message = message
        super message
      end
    end

    # link to itjobstack
    #@@url = 'http://www.itjobstack.com' 
    @@url = 'http://localhost:3000' 

    # sets link to itjobstack
    def url= url; @@url = url end
    module_function :url=

    # @return [String] link to itjobstack
    def url; @@url end
    module_function :url

    # returns [Hash] credentials for itjobstack
    def config; ::Itjobstack.config end
    module_function :config

    # @return [Boolean] true for validity of credentials
    def credentials?
      conn = Faraday.new url: url
      answer = conn.get '/api/v1/profiles/credentials/' do |req|
        req.params[:email]  = config[:email]
        req.params[:apiKey] = config[:apiKey]
      end.body
      result = JSON.parse answer
      raise Exception.new result["advice"] if result["response"] == 'notOk'
			true
    end
    module_function :credentials?

    # fetchs all jobs
    #
    # @return [Array] array with job json objects
    def fetch_jobs 
      conn = Faraday.new url: url
      answer = conn.get '/api/v1/employer/jobs/' do |req|
        req.params[:email]  = config[:email]
        req.params[:apiKey] = config[:apiKey]
      end.body
      result = JSON.parse answer
      raise Exception.new result["advice"] if result["response"] == 'notOk'
      JSON.parse result["packet"]["jobs"]
    end
    module_function :fetch_jobs

    # creates a job
    #
    # @param title [String] title of job
    # @param tags [Array] word tags of job
    # @param teaser [String] teaser of job
    # @param link [String] link of job
    # @param publish [Boolean] publication status of job
    def create_job title:, tags: [], teaser:, link:, publish: false
      conn = Faraday.new url: url
      answer = conn.post '/api/v1/employer/job/create' do |req|
        req.body = {email: config[:email], apiKey: config[:apiKey], title: title, tags: tags, teaser: teaser, link: link, publish: publish}
      end.body
      result = JSON.parse answer
      raise Exception.new result["advice"] if result["response"] == 'notOk'
      result["packet"]["_id"]
    end
    module_function :create_job

    # updates a job
    #
    # @param id [String] id of job to update
    # @param title [String] title of job
    # @param tags [Array] word tags of job
    # @param teaser [String] teaser of job
    # @param link [String] link of job
    # @param publish [Boolean] publication status of job
    # @raise [API::Exception] if update failed
    # @return [Boolean] true if job updated
    def update_job id:, title:, tags: [], teaser:, link:, publish: false
      conn = Faraday.new url: url
      answer = conn.put '/api/v1/employer/job/update' do |req|
        req.body = {email: config[:email], apiKey: config[:apiKey], _id: id, title: title, tags: tags, teaser: teaser, link: link, publish: publish}
      end.body
      result = JSON.parse answer
      raise Exception.new result["advice"] if result["response"] == 'notOk'
      true
    end
    module_function :update_job

    # deletes a job
    #
    # @param id [String] id of job to update
    # @raise [API::Exception] if delete failed
    def delete_job id: 
      conn = Faraday.new url: url
      answer = conn.delete '/api/v1/employer/job/delete' do |req|
        req.body = {email: config[:email], apiKey: config[:apiKey], _id: id}
      end.body
      return true if answer.empty?
      result = JSON.parse answer
      raise Exception.new result["advice"] if result["response"] == 'notOk'
    end
    module_function :delete_job

    # publishes a job
    #
    # @param id [String] id of job to publish 
    # @raise [API::Exception] if publish failed
    # @return [Boolean] true if job published 
    def publish_job id: 
      conn = Faraday.new url: url
      answer = conn.post '/api/v1/employer/job/publish' do |req|
        req.body = {email: config[:email], apiKey: config[:apiKey], _id: id, publish: true}
      end.body
      result = JSON.parse answer
      raise Exception.new result["advice"] if result["response"] == 'notOk'
			true
    end
    module_function :publish_job

    # unpublishes a job
    #
    # @param id [String] id of job to unpublish 
    # @raise [API::Exception] if unpublish failed
    # @return [Boolean] true if job unpublished 
    def unpublish_job id: 
      conn = Faraday.new url: url
      answer = conn.post '/api/v1/employer/job/publish' do |req|
        req.body = {email: config[:email], apiKey: config[:apiKey], _id: id, publish: false}
      end.body
      result = JSON.parse answer
      raise Exception.new result["advice"] if result["response"] == 'notOk'
			true
    end
    module_function :unpublish_job

    # uploads file to job 
    #
    # @param id [String] id of job to upload file to 
    # @raise [API::Exception] if file path is no file
    # @raise [API::Exception] if file was not updated
    # @return [Boolean] true if file uploaded
    def set_job_file id:, file:
      raise Exception.new "file don't exist - check file path given" unless File.exists? file
      conn = Faraday.new url: url do |f|
        f.request  :multipart
        f.request  :url_encoded
        f.adapter  :net_http
      end
      answer = conn.put '/api/v1/employer/job/spec/set' do |req|
        req.body = {email: config[:email], apiKey: config[:apiKey], _id: id, data: Faraday::UploadIO.new(file, 'pdf') }
      end.body
      result = JSON.parse answer
      raise Exception.new result["advice"] if result["response"] == 'notOk'
      true
    end
    module_function :set_job_file

    # removes file of job 
    #
    # @param id [String] id of job to remove file from
    # @raise [API::Exception] if file was not removed
    # @return [Boolean] true if file removed
    def unset_job_file id:
      conn = Faraday.new url: url
      answer = conn.put '/api/v1/employer/job/spec/unset' do |req|
        req.body = {email: config[:email], apiKey: config[:apiKey], _id: id }
      end.body
      result = JSON.parse answer
      raise Exception.new result["advice"] if result["response"] == 'notOk'
      true
    end
    module_function :unset_job_file
  end
end

